#ifndef __PLUGAPI_H_
#define __PLUGAPI_H_

/**
 * struct plugin_multiply_api
 *
 *   This is a struct of function definitions that represents one of the 
 *   available plugin APIs. This particular API is a multiplication API,
 *   which provides functions for multiplying numbers.
 *
 *   In Motion, one API could be an input API, a second could be an 
 *   output API, and a third could be an event API, for example.
 */
struct plugin_multiply_api
{
	int (*multiply)(int, int);   /* multiply two integers */
};

/**
 * struct plugin_divide_api
 *
 *   Similarly to the multiplication API, this struct of function definitions
 *   represents a division API, which provides functions for dividing numbers.
 */
struct plugin_divide_api
{
	int (*divide)(int, int);     /* divide two integers */
};

/**
 * plugin_api
 *
 *   This is a union of all the previously defined APIs. A plugin adheres to
 *   one particular API, but having a union allows us to have a plugin info
 *   struct (see below) that has only one API member.
 */
typedef union _plugin_api
{
	struct plugin_multiply_api mul_api; /* the multiplication API */
	struct plugin_divide_api div_api;   /* the division API */
} plugin_api;

/**
 * Plugin flags - the two defined below controls which API the plugin adheres
 * to, but other flags could be defined as well, of course.
 */
#define M_PLUGIN_MULTIPLY 1    /* plugin adheres to the multiplication API */
#define M_PLUGIN_DIVIDE   2    /* plugin adheres to the division API */

/**
 * struct motion_plugin
 *
 *   Struct for storing plugin info. There are three important parts:
 *
 *   1. Plugin data - name, version etc. Could add more fields here, e.g.
 *      for copyright data.
 *   2. Flags - plugin flags (see above).
 *   3. API - the union defined above.
 *
 *   The idea is that a plugin defines a static instance of this struct
 *   with a specific name (see below), and the plugin loader checks for the
 *   presence of the instance (it's mandatory) and can return it to the 
 *   application.
 */
struct motion_plugin
{
	char *name;          /* name of the plugin */

	char *version;       /* plugin version as a string */

	char *author;        /* plugin author */

	char *homepage;      /* plugin homepage (is this relevant?) */

	unsigned int flags;  /* plugin flags, see above */

	plugin_api api;	     /* the API union */
};

/**
 * struct mhandle
 *
 *   Plugin handle returned and used by the plugin-handling functions. The
 *   struct is opaque, meaning that it is not intended to be used directly
 *   by the application (other than as a handle).
 */
struct mhandle;

/**
 * Symbols to look for in the plugin. The first two define functions that the
 * plugin-handling code calls automatically (if they exist) when the plugin is
 * loaded and unloaded, respectively. The third defines the name of the 
 * mandatory plugin info struct.
 */
#define M_LOAD_FUNC "motion_plugin_load"
#define M_UNLOAD_FUNC "motion_plugin_unload"
#define M_INFO_SYMBOL "motion_plugin_info"

/**
 * Function definitions for the load and unload functions. These are currently
 * only used by the plugin-handling functions, but could possibly be useful for
 * a plugin...
 */
typedef int (*plugin_load_function)(void);
typedef void (*plugin_unload_function)(void);

#endif /* __PLUGAPI_H_ */

